/*
 * SPDX-FileCopyrightText: 2018-2018 CSSlayer <wengxt@gmail.com>
 *
 * SPDX-License-Identifier: LGPL-2.1-or-later
 *
 */

#include "log.h"

namespace fcitx {

Q_LOGGING_CATEGORY(migrator, "migrator")

} // namespace fcitx
