/*
 * SPDX-FileCopyrightText: 2018-2018 CSSlayer <wengxt@gmail.com>
 *
 * SPDX-License-Identifier: LGPL-2.1-or-later
 *
 */
#ifndef _PINYINDICTMANAGER_LOG_H_
#define _PINYINDICTMANAGER_LOG_H_

#include <QLoggingCategory>

namespace fcitx {

Q_DECLARE_LOGGING_CATEGORY(migrator)

} // namespace fcitx

#endif // _PINYINDICTMANAGER_LOG_H_
