/*
 * SPDX-FileCopyrightText: 2017~2017 CSSlayer <wengxt@gmail.com>
 *
 * SPDX-License-Identifier: GPL-2.0-or-later
 */
#ifndef _KCM_FCITX_LOGGING_H_
#define _KCM_FCITX_LOGGING_H_

#include <QLoggingCategory>

Q_DECLARE_LOGGING_CATEGORY(KCM_FCITX5)

#endif // _KCM_FCITX_LOGGING_H_
